/*
 * Copyright  2001-2002 Jussi Kivilinna <hullu@unitedadmins.com>
 *
 *    This file is part of cs-rw Metamod Plugin.
 *
 *    I think this file should be is GPL because Metamod is GPL .. but I allow you to
 *    use any part that isn't from Metamod or Valve's HLSDK for whatever you want :)
 */

#include "cs-rw.h"


//players array .. index 0 is used for invalid entities
CPlayer players[33];


void CPlayer::PutInServer(edict_t *pThis)
{
	// If index is 0 pThis is invalid :(
	// Don't think proxies as players
	if(m_index==0 || pThis->v.flags & FL_PROXY) 
	{
		m_connected = false;
		m_pEdict    = NULL;
		
		SetMetaResult(MRES_IGNORED);
		return;
	}
	
	//Set defaults
	m_connected = true;
	m_pEdict    = pThis;
	
	//has not respawned yet!
	m_OldHealth = -50.0f;
	m_RespawnDetected = false;

	SetMetaResult(MRES_HANDLED);
	return;
}


void CPlayer::Disconnect()
{
	//Mark disconnected
	m_connected = false;
	m_pEdict = NULL;
	
	//doh :)
	m_OldHealth = -50.0f;
	m_RespawnDetected = false;

	SetMetaResult(MRES_HANDLED);
	return;
}


void CPlayer::RemoveWeapons()
{
	//Remove weapons from player using stripper entity
	if(csrw.pWeaponStripper) MDLL_Use(csrw.pWeaponStripper,m_pEdict);
}


#define	SF_NORESPAWN	( 1 << 30 ) // !!!set this bit on guns and stuff that should never respawn.

void CPlayer::AddItems(const char *item_list)
{
	if(!item_list || !*item_list) return;
	
	char *a = strdup(item_list);
	char *str = a;
	char *next;
	
	do {
		//First find next ';'
		next = strchr(str,';');
		
		if(next)
		{
			//make it null-terminator
			*next= 0;
		
			//Next for next run
			next++;
		}
		
		//Item-insertion code based on BMX plugin's version
		//Only weapons/ammos/items please :)
		if( !( strncmp(str,"weapon_",7) && strncmp(str,"ammo_",5) && strncmp(str,"item_",5) ) )
		{
			//Create item
			int istr = MAKE_STRING(str);
			edict_t* pItem = CREATE_NAMED_ENTITY(istr);
			
			//Null .. something went wrong there :/
			if( !FNullEnt(pItem) )
			{
				//Set right origin
				pItem->v.origin = m_pEdict->v.origin;
			
				//Don't allow item to respawn
				pItem->v.spawnflags |= SF_NORESPAWN;
			
				//Init item gamedll code
				MDLL_Spawn(pItem);
			
				//Touch player with item (give item to player)
				MDLL_Touch(pItem,m_pEdict);
			}
		}

		//Set str for next run
		str = next;

	} while(str && *str);
	
	free(a);
}


const char *CPlayer::GetTeamItems()
{
	//Model!
	char *model_name = (g_engfuncs.pfnInfoKeyValue((*g_engfuncs.pfnGetInfoKeyBuffer)( m_pEdict ), "model"));
	
	if(!model_name) return NULL;
	
	if ((!strcmp(model_name, "urban")) ||  // Seal Team 6
	    (!strcmp(model_name, "gsg9"))  ||  // German GSG-9
	    (!strcmp(model_name, "sas"))   ||  // UK SAS
	    (!strcmp(model_name, "gign"))  ||  // French GIGN
	    (!strcmp(model_name, "vip")) )     // VIP
	{
		return CVAR_GET_STRING("csrw_ct_weapons");
	} 
	else if ((!strcmp(model_name, "terror"))   ||  // Phoenix Connektion
	         (!strcmp(model_name, "arab"))     ||  // old L337 Krew
	    	 (!strcmp(model_name, "leet"))     ||  // L337 Krew
	    	 (!strcmp(model_name, "arctic"))   ||  // Arctic Avenger 
	    	 (!strcmp(model_name, "guerilla")) )   // Gorilla Warfare
	{
		return CVAR_GET_STRING("csrw_t_weapons");
	}
		
	return NULL;
}


void CPlayer::PostThink()
{
	//All return HANDLED in here so set MetaResult here
	SetMetaResult(MRES_HANDLED);


	//Respawn was detected on last think
	if(m_RespawnDetected)
	{
		m_RespawnDetected = false;
		
		//Get items
		const char* items = GetTeamItems();
		
		if(items)
		{
			//Remove items/weapons
			RemoveWeapons();
		
			//Add new items/weapons/ammos
			AddItems(items);
		}
	}


	//Check if was dead
	if(m_OldHealth<=0.0f)
	{
		//Was dead on last think..
		
		//Check if still dead
		if(IsAlive())
		{
			//Respawn detected!!
			m_RespawnDetected = true;
		}
	}


	//Save old health
	m_OldHealth = CurrentHealth();


	return;
}


