/*
 * Copyright  2001-2002 Jussi Kivilinna <hullu@unitedadmins.com>
 *
 *    This file is part of cs-rw Metamod Plugin.
 *
 *    I think this file should be is GPL because Metamod is GPL .. but I allow you to
 *    use any part that isn't from Metamod or Valve's HLSDK for whatever you want :)
 */

#include "cs-rw.h"


csrw_t	csrw;


static void ClientPutInServer( edict_t *pEdict )
{
	if (!gpGlobals->deathmatch) RETURN_META(MRES_IGNORED);
	
	//Get player pointer
	CPlayer *pPlayer = CPlayer::GetPlayerFromArray(pEdict);
	
	//call player class function
	pPlayer->PutInServer(pEdict);
	
	RETURN_META(pPlayer->GetMetaResult());
}


static void ClientDisconnect( edict_t *pEdict )
{
	if (!gpGlobals->deathmatch) RETURN_META(MRES_IGNORED);
	
	//Get player pointer
	CPlayer *pPlayer = CPlayer::GetPlayerFromArray(pEdict);
	
	if(!pPlayer->m_connected)
	{
		//Not valid!
		RETURN_META(MRES_IGNORED);
	}

	//call player class function
	pPlayer->Disconnect();
	
	RETURN_META(pPlayer->GetMetaResult());
}


static void PlayerPostThink( edict_t *pEdict )
{	
	if (!gpGlobals->deathmatch || !csrw.active) RETURN_META(MRES_IGNORED);
	
	//Get player pointer
	CPlayer *pPlayer = CPlayer::GetPlayerFromArray(pEdict);
	
	if(!pPlayer->m_connected)
	{
		if(!pEdict || !pEdict->v.flags & FL_FAKECLIENT)
		{
			//Not valid!
			RETURN_META(MRES_IGNORED);
		}
		
		//Metamod bot fix
		
		//call player class function
		pPlayer->PutInServer(pEdict);
	}

	//call player class function
	pPlayer->PostThink();
	
	RETURN_META(pPlayer->GetMetaResult());
}


static float next_checktime=0;


static void StartFrame(void)
{
	if (!gpGlobals->deathmatch) RETURN_META(MRES_IGNORED);

	if (next_checktime<gpGlobals->time)
	{
		//Get bool values
		csrw.active	= (int)CVAR_GET_FLOAT("csrw_on") > 0.01f;
		
		next_checktime	= gpGlobals->time+0.5f;
	}
	
	RETURN_META(MRES_HANDLED);
}


static int DispatchSpawn( edict_t *pent )
{	
	if (!gpGlobals->deathmatch) RETURN_META_VALUE(MRES_IGNORED, 0);
	
	if (!stricmp(STRING(pent->v.classname), "worldspawn"))
	{
		//Clear players
		for (int pid=0;pid<33;pid++) 
		{
			players[pid].m_connected	= false;
			players[pid].m_pEdict		= NULL;
			players[pid].m_index		= pid;
		}
		
		
		//Create weapon stripper
		csrw.pWeaponStripper = CREATE_NAMED_ENTITY(MAKE_STRING("player_weaponstrip"));
		MDLL_Spawn(csrw.pWeaponStripper);
 
 
		next_checktime = gpGlobals->time + (-0.01f);
	}
	
	RETURN_META_VALUE(MRES_HANDLED,	0);
	return 0;
}


static void SrvDeactive(void)
{
	if (!gpGlobals->deathmatch) RETURN_META(MRES_IGNORED);
	
	csrw.pWeaponStripper = NULL;
	
	RETURN_META(MRES_HANDLED);
}


static DLL_FUNCTIONS gFunctionTable = 
{
	NULL,						//! pfnGameInit()
	DispatchSpawn,					//! pfnSpawn()
	NULL,						//! pfnThink()
	NULL,						//! pfnUse()
	NULL,						//! pfnTouch()
	NULL,						//! pfnBlocked()
	NULL,						//! pfnKeyValue()
	NULL,						//! pfnSave()
	NULL,						//! pfnRestore()
	NULL,						//! pfnSetAbsBox()

	NULL,						//! pfnSaveWriteFields()
	NULL,						//! pfnSaveReadFields()

	NULL,						//! pfnSaveGlobalState()
	NULL,						//! pfnRestoreGlobalState()
	NULL,						//! pfnResetGlobalState()

	NULL,						//! pfnClientConnect()
	ClientDisconnect,				//! pfnClientDisconnect()
	NULL,						//! pfnClientKill()
	ClientPutInServer,				//! pfnClientPutInServer()
	NULL,						//! pfnClientCommand()
	NULL,						//! pfnClientUserInfoChanged()
	NULL,						//! pfnServerActivate()
	SrvDeactive,					//! pfnServerDeactivate()
	
	NULL,						//! pfnPlayerPreThink()
	PlayerPostThink,				//! pfnPlayerPostThink()

	StartFrame,					//! pfnStartFrame()
	NULL,						//! pfnParmsNewLevel()
	NULL,						//! pfnParmsChangeLevel()

	NULL,						//! pfnGetGameDescription()
	NULL,						//! pfnPlayerCustomization()

	NULL,						//! pfnSpectatorConnect()
	NULL,						//! pfnSpectatorDisconnect()
	NULL,						//! pfnSpectatorThink()
	
	NULL,						//! pfnSys_Error()

	NULL,						//! pfnPM_Move()
	NULL,						//! pfnPM_Init()
	NULL,						//! pfnPM_FindTextureType()
	
	NULL,						//! pfnSetupVisibility()
	NULL,						//! pfnUpdateClientData()
	NULL,						//! pfnAddToFullPack()
	NULL,						//! pfnCreateBaseline()
	NULL,						//! pfnRegisterEncoders()
	NULL,						//! pfnGetWeaponData()
	NULL,						//! pfnCmdStart()
	NULL,						//! pfnCmdEnd()
	NULL,						//! pfnConnectionlessPacket()
	NULL,						//! pfnGetHullBounds()
	NULL,						//! pfnCreateInstancedBaselines()
	NULL,						//! pfnInconsistentFile()
	NULL,						//! pfnAllowLagCompensation()
};

C_DLLEXPORT int	GetEntityAPI2( DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion )
{
	LOG_DEVELOPER(PLID, "called: GetEntityAPI2; version=%d", *interfaceVersion);
	if(!pFunctionTable) {
		LOG_ERROR(PLID,	"GetEntityAPI2 called with null	pFunctionTable");
		return(FALSE);
	}
	else if(*interfaceVersion != INTERFACE_VERSION)	{
		LOG_ERROR(PLID,	"GetEntityAPI2 version mismatch; requested=%d ours=%d",	*interfaceVersion, INTERFACE_VERSION);
		//! Tell engine	what version we	had, so	it can figure out who is out of	date.
		*interfaceVersion = INTERFACE_VERSION;
		return(FALSE);
	}
	memcpy(pFunctionTable, &gFunctionTable,	sizeof(DLL_FUNCTIONS));

	return(TRUE);
}


//Bot PutInServer fix up
static edict_t * CreateFakeClient_Post(const char *netname)
{
	if (!gpGlobals->deathmatch) RETURN_META_VALUE(MRES_IGNORED,NULL);
	
	edict_t *fake_client=META_RESULT_ORIG_RET(edict_t *);
	
	if(FNullEnt(fake_client))
		RETURN_META_VALUE(MRES_IGNORED,NULL);
		
	//Get player pointer
	CPlayer *pPlayer = CPlayer::GetPlayerFromArray(fake_client);
	
	pPlayer->PutInServer(fake_client);
	
	RETURN_META_VALUE(pPlayer->GetMetaResult(),NULL);
	return NULL;
}


C_DLLEXPORT int GetEngineFunctions_Post(enginefuncs_t *pengfuncsFromEngine_Post, int *interfaceVersion ) 
{
	if(!pengfuncsFromEngine_Post) {
		UTIL_LogPrintf("%s: GetEngineFunctions_Post called with null pengfuncsFromEngine_Post", Plugin_info.logtag);
		return(FALSE);
	}
	else if(*interfaceVersion != ENGINE_INTERFACE_VERSION) {
		UTIL_LogPrintf("%s: GetEngineFunctions_Post version mismatch; requested=%d ours=%d", Plugin_info.logtag, *interfaceVersion, ENGINE_INTERFACE_VERSION);
		// Tell metamod what version we had, so it can figure out who is out of date.
		*interfaceVersion = ENGINE_INTERFACE_VERSION;
		return(FALSE);
	}
	
	memset(pengfuncsFromEngine_Post, 0, sizeof(enginefuncs_t));
	
	pengfuncsFromEngine_Post->pfnCreateFakeClient = CreateFakeClient_Post;
	
	return TRUE;
}
