/*
 * Copyright  2001-2002 Jussi Kivilinna <hullu@unitedadmins.com>
 *
 *    This file is part of cs-rw Metamod Plugin.
 *
 *    You are allowed to copy/modify this file freely. I don't care :)
 */

#include "cs-rw.h"


// Plugin startup.  Register commands and cvars.
static cvar_t csrw_cvars[] = {
	{"csrw_on",			"1",						FCVAR_EXTDLL, 1, NULL},
	{"csrw_t_weapons",		"weapon_knife;weapon_glock18;ammo_9mm",		FCVAR_EXTDLL, 0, NULL},
	{"csrw_ct_weapons",		"weapon_knife;weapon_usp;ammo_9mm",     	FCVAR_EXTDLL, 0, NULL},

	{"csrw_version",		VVERSION,FCVAR_EXTDLL|FCVAR_SERVER, 0, NULL},
	
	{NULL,NULL,0,0,NULL},
};

#include <ctype.h>

inline char *___strlwr(char *s)
{
	if (s) 
	{
		for (unsigned long slen = strlen(s),i=0;i<slen;i++) 
			s[i]=tolower(s[i]);
	}
	return s;
}


inline char *strstri(const char *s1, const char *s2)
{
	//Have to strdup first .. because we don't want to lower playernames :)
	const char *low_s1=___strlwr(strdup(s1));
	const char *low_s2=___strlwr(strdup(s2));

	const char *point = strstr(low_s1,low_s2);
	
	if (point != NULL) point = s1+(point-low_s1);
	
	free((void *)low_s1);
	free((void *)low_s2);
	
	return (char *)point;
}


static void CSRWGive(void)
{
	//cs-rw_give <pname|pid> <items;weapons;ammo>
	if(CMD_ARGC()<3) return;
	
	const char *player_ident = CMD_ARGV(1);
	const char *items = CMD_ARGV(2);
	int pid=-1;
	
	//check
	if (!player_ident || !*player_ident || !items || !*items) return;

	//All Players
	if(*player_ident=='*')
	{
		if(*(player_ident+1)=='c')
		{
			//CounterTerrorists
			for(pid=1;pid<=gpGlobals->maxClients;pid++)
			{
				if(!players[pid].m_connected) continue;
				if(!players[pid].IsAlive()) continue;

				char *model_name = (g_engfuncs.pfnInfoKeyValue((*g_engfuncs.pfnGetInfoKeyBuffer)( players[pid].m_pEdict ), "model"));

				if ((!strcmp(model_name, "urban")) ||  // Seal Team 6
				    (!strcmp(model_name, "gsg9"))  ||  // German GSG-9
				    (!strcmp(model_name, "sas"))   ||  // UK SAS
				    (!strcmp(model_name, "gign"))  ||  // French GIGN
				    (!strcmp(model_name, "vip")) )     // VIP
				{
					players[pid].AddItems(items);		
				}
			}
		
			ALERT(at_console,"[" VLOGTAG "] Gave to counter-terrorists: \"%s\"!\n",items);
			
			return;			
		}
		
		if(*(player_ident+1)=='t')
		{
			//CounterTerrorists
			for(pid=1;pid<=gpGlobals->maxClients;pid++)
			{
				if(!players[pid].m_connected) continue;
				if(!players[pid].IsAlive()) continue;
				
				char *model_name = (g_engfuncs.pfnInfoKeyValue((*g_engfuncs.pfnGetInfoKeyBuffer)( players[pid].m_pEdict ), "model"));
				
				if ((!strcmp(model_name, "terror"))   ||  // Phoenix Connektion
			            (!strcmp(model_name, "arab"))     ||  // old L337 Krew
			    	    (!strcmp(model_name, "leet"))     ||  // L337 Krew
			    	    (!strcmp(model_name, "arctic"))   ||  // Arctic Avenger 
			    	    (!strcmp(model_name, "guerilla")) )   // Gorilla Warfare
				{
					players[pid].AddItems(items);
				}
			}
			
			ALERT(at_console,"[" VLOGTAG "] Gave to terrorists: \"%s\"!\n",items);
			
			return;			
		}
		
		//All players
		for(pid=1;pid<=gpGlobals->maxClients;pid++)
		{
			if(!players[pid].m_connected) continue;
			if(!players[pid].IsAlive()) continue;
				
			players[pid].AddItems(items);		
		}
		
		ALERT(at_console,"[" VLOGTAG "] Gave to all players: \"%s\"!\n",items);
		
		return;
	}

	//Player ID
	if(*player_ident=='#')
	{
		if(!*(player_ident+1)) return;

		sscanf((player_ident+1),"%d",&pid);

		if(pid<1 || pid>gpGlobals->maxClients) return;
		if(!players[pid].m_connected) return;
		if(!players[pid].IsAlive()) return;
		
		goto FoundPlayerAndSetItToPID;
	}
	
	//Check for exact name matches
	for(pid=1;pid<=gpGlobals->maxClients;pid++)
	{
		if(!players[pid].m_connected) continue;
		if(!players[pid].IsAlive()) continue;
		
		if(stricmp(player_ident,STRING(players[pid].m_pEdict->v.netname))!=0) continue;
		
		goto FoundPlayerAndSetItToPID;
	}
	
	//Check for partial name matches
	for(pid=1;pid<=gpGlobals->maxClients;pid++)
	{
		if(!players[pid].m_connected) continue;
		if(!players[pid].IsAlive()) continue;
		
		int slen = strlen(STRING(players[pid].m_pEdict->v.netname));
		if(strnicmp(STRING(players[pid].m_pEdict->v.netname),player_ident,slen)!=0) continue;
		
		goto FoundPlayerAndSetItToPID;
	}
	
	//Check for any matches
	for(pid=1;pid<=gpGlobals->maxClients;pid++)
	{
		if(!players[pid].m_connected) continue;
		if(!players[pid].IsAlive()) continue;
		
		if(strstri(STRING(players[pid].m_pEdict->v.netname),player_ident)==NULL) continue;
		
		goto FoundPlayerAndSetItToPID;
	}
	
	return;
	
FoundPlayerAndSetItToPID:
	
	ALERT(at_console,"[" VLOGTAG "] Gave to player: '%s'  <  \"%s\"!\n",STRING(players[pid].m_pEdict->v.netname),items);
	players[pid].AddItems(items);

	return;	
}


void CSRWList(void)
{
	ALERT(at_console," ID\t\tStatus\t\tName\n");
	for(int pid=1;pid<=gpGlobals->maxClients;pid++)
	{
		ALERT(at_console," #%d\t\t%s\t\t%s\n",
			pid,
			players[pid].m_connected?(players[pid].IsAlive()?"Alive":"Dead"):"     ",
			players[pid].m_connected?STRING(players[pid].m_pEdict->v.netname):"[not connected]"
		);
	}
}


void plugin_init(void) 
{
	LOG_MESSAGE(PLID, "%s v%s, %s", Plugin_info.name, Plugin_info.version, Plugin_info.date);
	LOG_MESSAGE(PLID, "by %s", Plugin_info.author);
	LOG_MESSAGE(PLID, "   %s", Plugin_info.url);
	LOG_MESSAGE(PLID, "compiled: "__DATE__ "," __TIME__" EET");
	
	
	//Register list of cvars
	for(int i = 0; csrw_cvars[i].name; i++ )
		CVAR_REGISTER( &csrw_cvars[i] );
		
	REG_SVR_COMMAND("csrw_give",CSRWGive);
	REG_SVR_COMMAND("csrw_list",CSRWList);

	csrw.active = true;
}











